/*
 * JFLex spec for simple calculator
 */

// Section one .. user code that is copied verbatim into the top of the generated lexer file

%%

// Section two ... options and declare constants
%class Lexer     // Name of the class to produce with this flex def
%unicode         // Defines the charset to be scanned
%line            // Switch on line counting
%column          // Switch on column counting
%type Token      // specify the return type of the tokens

%{
    // Code here gets copied verbatim to the beginning of the Lexer class

    // helper functions to create tokens
    private Token token(Tok type) {
        return new Token(type, yyline+1, yycolumn+1);
    }

    private Token token(Tok type, Object value) {
        return new Token(type, yyline+1, yycolumn+1, value);
    }
%}

// macro/constants regex for different token types
Identifier = [a-z][_A-Za-z0-9]*
IntegerLiteral = [0-9]+
FloatLiteral = [0-9]+"." | "."[0-9]+ | [0-9]+"."[0-9]+
Whitespace = [ \t\n]

%%

// Section three ... lexical analysis rules
<YYINITIAL> {
    "+"                 { return token(Tok.PLUS); }
    "-"                 { return token(Tok.MINUS); }
    "*"                 { return token(Tok.TIMES); }
    "/"                 { return token(Tok.DIVIDE); }
    ","                 { return token(Tok.COMMA); }
    "("                 { return token(Tok.LPAREN); }
    ")"                 { return token(Tok.RPAREN); }

    {Identifier}        { return token(Tok.IDENT, yytext()); }
    {IntegerLiteral}    { return token(Tok.INT, yytext()); }
    {FloatLiteral}      { return token(Tok.FLOAT, yytext()); }

    {Whitespace}        { /* ignore */ }
}